@@ -8,11 +8,7 @@ from vyper import ast as vy_ast
 from vyper import builtin_functions as vy_fn
 
 st_decimals = st.decimals(
-    min_value=-(2 ** 32),
-    max_value=2 ** 32,
-    allow_nan=False,
-    allow_infinity=False,
-    places=10,
+    min_value=-(2 ** 32), max_value=2 ** 32, allow_nan=False, allow_infinity=False, places=10
 )
 
 
