@@ -13,7 +13,6 @@ from eth_abi import decode_single, encode_single
 from vyper.codegen.types import (
     BASE_TYPES,
     INTEGER_TYPES,
-    SIGNED_INTEGER_TYPES,
     parse_bytes_m_info,
     parse_decimal_info,
     parse_integer_typeinfo,
@@ -23,6 +22,7 @@ from vyper.utils import (
     DECIMAL_DIVISOR,
     SizeLimits,
     checksum_encode,
+    int_bounds,
     is_checksum_encoded,
     round_towards_zero,
     unsigned_to_signed,
@@ -466,22 +466,6 @@ def test_convert() -> {o_typ}:
 
     c1_exception = None
     skip_c1 = False
-    if i_typ in INTEGER_TYPES and o_typ in INTEGER_TYPES - {"uint256"}:
-        # Skip conversion of positive integer literals because compiler reads them
-        # as target type.
-        if val >= 0:
-            c1_exception = InvalidType
-
-    if i_typ in SIGNED_INTEGER_TYPES and o_typ in SIGNED_INTEGER_TYPES and val < 0:
-        # similar, skip conversion of negative integer literals because compiler
-        # infers them as target type.
-        c1_exception = InvalidType
-
-    if i_typ.startswith(("int", "uint")) and o_typ.startswith("bytes"):
-        # integer literals get upcasted to uint256 / int256 types, so the convert
-        # will not compile unless it is bytes32
-        if o_typ != "bytes32":
-            c1_exception = TypeMismatch
 
     # Skip bytes20 literals when there is ambiguity with `address` since address takes precedence.
     # generally happens when there are only digits in the literal.
@@ -530,13 +514,60 @@ def test_memory_variable_convert(x: {i_typ}) -> {o_typ}:
     assert c4.test_memory_variable_convert(val) == expected_val
 
 
+@pytest.mark.parametrize("typ", ["uint8", "int128", "int256", "uint256"])
+@pytest.mark.parametrize("val", [1, 2, 2 ** 128, 2 ** 256 - 1, 2 ** 256 - 2])
+def test_enum_conversion(get_contract_with_gas_estimation, assert_compile_failed, val, typ):
+    roles = "\n    ".join([f"ROLE_{i}" for i in range(256)])
+    contract = f"""
+enum Roles:
+    {roles}
+
+@external
+def foo(a: Roles) -> {typ}:
+    return convert(a, {typ})
+
+@external
+def bar(a: uint256) -> Roles:
+    return convert(a, Roles)
+    """
+    if typ == "uint256":
+        c = get_contract_with_gas_estimation(contract)
+        assert c.foo(val) == val
+        assert c.bar(val) == val
+    else:
+        assert_compile_failed(lambda: get_contract_with_gas_estimation(contract), TypeMismatch)
+
+
+@pytest.mark.parametrize("typ", ["uint8", "int128", "int256", "uint256"])
+@pytest.mark.parametrize("val", [1, 2, 3, 4, 2 ** 128, 2 ** 256 - 1, 2 ** 256 - 2])
+def test_enum_conversion_2(
+    get_contract_with_gas_estimation, assert_compile_failed, assert_tx_failed, val, typ
+):
+    contract = f"""
+enum Status:
+    STARTED
+    PAUSED
+    STOPPED
+
+@external
+def foo(a: {typ}) -> Status:
+    return convert(a, Status)
+    """
+    if typ == "uint256":
+        c = get_contract_with_gas_estimation(contract)
+        lo, hi = int_bounds(signed=False, bits=3)
+        if lo <= val <= hi:
+            assert c.foo(val) == val
+        else:
+            assert_tx_failed(lambda: c.foo(val))
+    else:
+        assert_compile_failed(lambda: get_contract_with_gas_estimation(contract), TypeMismatch)
+
+
 # TODO CMC 2022-04-06 I think this test is somewhat unnecessary.
 @pytest.mark.parametrize(
     "builtin_constant,out_type,out_value",
-    [
-        ("ZERO_ADDRESS", "bool", False),
-        ("msg.sender", "bool", True),
-    ],
+    [("ZERO_ADDRESS", "bool", False), ("msg.sender", "bool", True)],
 )
 def test_convert_builtin_constant(
     get_contract_with_gas_estimation, builtin_constant, out_type, out_value
