@@ -197,18 +197,6 @@ def callMeMaybe() -> (Bytes[100], uint256, Bytes[20]):
     assert c.callMeMaybe() == [b"here is my number", 555123456, b"baby"]
 
 
-def test_builtin_constants_as_default(get_contract):
-    code = """
-@external
-def foo(a: int128 = MIN_INT128, b: int128 = MAX_INT128) -> (int128, int128):
-    return a, b
-    """
-    c = get_contract(code)
-    assert c.foo() == [-(2 ** 127), 2 ** 127 - 1]
-    assert c.foo(31337) == [31337, 2 ** 127 - 1]
-    assert c.foo(13, 42) == [13, 42]
-
-
 def test_environment_vars_as_default(get_contract):
     code = """
 xx: uint256
@@ -302,6 +290,20 @@ struct Bar:
 @external
 def foo(bar: Bar = Bar({a: msg.sender, b: Baz({c: block.coinbase, d: -10})})): pass
     """,
+    """
+A: public(address)
+
+@external
+def foo(a: address = empty(address)):
+    self.A = a
+    """,
+    """
+A: public(int112)
+
+@external
+def foo(a: int112 = min_value(int112)):
+    self.A = a
+    """,
 ]
 
 
