@@ -1,5 +1,7 @@
 from typing import Tuple
 
+from vyper.codegen.types import INTEGER_TYPES, SIGNED_INTEGER_TYPES, UNSIGNED_INTEGER_TYPES
+
 
 class AbstractDataType:
     """
@@ -41,42 +43,49 @@ class ArrayValueAbstractType(AbstractDataType):
 
 
 class BytesAbstractType(AbstractDataType):
-    """Abstract data class for bytes types (bytes32, bytes[])."""
+    """Abstract data class for bytes types (bytes, bytesM)."""
 
     _description = "bytes"
     _id = "bytes"
 
 
+class BytesMAbstractType(BytesAbstractType):
+    """Abstract data class for fixed bytes types (bytes32, bytes4, ...)"""
+
+    _description = "bytes<M> where M<=32"
+
+
 class NumericAbstractType(AbstractDataType):
     """
     Abstract data class for numeric types (capable of arithmetic).
     """
 
     _description = "numeric value"
-    _id_list: Tuple = ("int128", "int256", "decimal", "uint8", "uint256")
+    _id_list: Tuple = ("decimal",) + tuple(INTEGER_TYPES)
 
 
 class IntegerAbstractType(NumericAbstractType):
     """Abstract data class for integer numeric types (signed and unsigned)."""
 
     _description = "integer"
-    _id_list: Tuple = ("int128", "int256", "uint8", "uint256")
+    _id_list: Tuple = tuple(INTEGER_TYPES)
 
 
 class SignedIntegerAbstractType(IntegerAbstractType):
     """Abstract data class for signed integer numeric types."""
 
     _description = "signed integer"
-    _id_list: Tuple = ("int128", "int256")
+    _id_list: Tuple = tuple(SIGNED_INTEGER_TYPES)
 
 
 class UnsignedIntegerAbstractType(IntegerAbstractType):
     """Abstract data class for unsigned integer numeric types."""
 
     _description = "unsigned integer"
-    _id_list: Tuple = ("uint8", "uint256")
+    _id_list: Tuple = tuple(UNSIGNED_INTEGER_TYPES)
 
 
+# TODO rename me to DecimalAbstractType (confusion with BytesM - fixed bytes)
 class FixedAbstractType(NumericAbstractType):
     """
     Abstract data class for decimal numeric types.
